@@ -1,7 +1,15 @@
 /*
-   Time-stamp: <95/12/07 00:50:10 yusuf>
+   Time-stamp: <96/08/04 19:27:32 yusuf>
+
+   $Id: backup.c,v 1.30 1996/08/05 19:01:59 yusuf Exp $	
+
 */
 
+#ifndef lint
+static char vcid[] = "$Id: backup.c,v 1.30 1996/08/05 19:01:59 yusuf Exp $";
+#endif /* lint */
+
+
 
 
 /* Tape structure
@@ -16,8 +24,8 @@
  * 
  * Volume
  * ------
- * long volume_magic
- * long no_in_vol
+ * 
+ * Volume header information
  * 
  * DATA
  *               : file_info
@@ -47,8 +55,9 @@
  *   checksum
  *
  * 
- * Information file structure
+ * Information file structure 
  * 
+ * long magic
  * long archive_id                        )
  * int number_tapes                       )   in structure
  * long info_file_size                    )
@@ -57,16 +66,29 @@
  * long no_in_archive                     ) 
  * char archive_title                     )  
  * 
+ * IF COMPRESSED long size_compressed
+ * 
  * For each file:
  *       file_info
  *       file_name
  * 
- * Volume header information
  * 
- * At the end of the file
+ * Volume details
+ *   vol header                   )   for each volume
+ *   files selected               )   on archive
+ * 
+ * Volume/tape information
  *       volume    tape it's on   - for each volume
  * 
+ * Tape size information
+ *       tape      size in blocks
  * 
+ * 
+ * The info file is written in little endian format.
+ * 
+ * The info file can be compressed (using internal gzip).
+ * If it is, then there is an additional field - length of compressed
+ * info file
 */
 
 
@@ -74,104 +96,19 @@
 #include "backup.h"
 
 FILE *fdfifo;
-char fifo_name[MAXNAMLEN];
+char fifo_name[MAX_FNAME];
 
-
-_errstat my_gzip(char *file, char *wf)
-{
-    int ifd, ofd;
-    char l[1024];
-
-    sprintf(l, "I:Internal gzipping %s to %s", file, wf);
-    if (log_level > 3) write_log(l);
-    ifd = open(file, O_RDONLY);
-    if (ifd==-1) {
-	sprintf(l, "I:%s while opening file", strerror(errno));
-	write_log(l);
-	return -1;
-    }
-    ofd = creat(wf, S_IREAD|S_IWRITE);
-    if (ofd==-1) {
-	sprintf(l, "I:%s while creating file", strerror(errno));
-	write_log(l);
-	close(ifd); 
-	return -1;
-    }
-    zip(ifd, ofd);
-    close(ifd); close(ofd);
-    return 0;
-}
-
-        
-_errstat my_compress(char *file, char *wf)
-{
-/* Compresses the 'file' to output file 'wf' 
- * 
- * To aid speed, reads the whole file into memory and compresses
- * 
- * Returns 0 if OK, -1 if error, -2 if compressed file would be larger
- * 
- * Returns an error if given a NULL file
- * If the file is > COMPRESS_MAX_ORG (0x700000000), then an error
- *   is returned - I can't ever see a file this long - and if it is,
- *   it's probably best not to compress
- */
-    int fd;
-    struct stat b;
-    ULONG   c_len;
-    char    l[1024];
-
-    sprintf(l, "I:Original compressing %s to %s", file, wf);
-    if (log_level > 3) write_log(l);
-    if ((comp_buffer1 == NULL) || (comp_buffer2 == NULL))
-      return -1;
-    fd=open(file, O_RDONLY); 
-    if (fd == -1) 
-	return -1;
-    if (fstat(fd, &b) == -1) {close(fd); return -1;} 
-    if (b.st_size == 0) {close(fd); return -1;}	 /* not going to compress empty files */
-    if (b.st_size > COMPRESS_MAX_COM) {close(fd); return -1;}   /* too large for compress to handle */
-    if (b.st_size > COMPRESS2_BUFFER_SIZE) {close(fd); return -1;}   /* too large for compress buffer */
-    if (read(fd, comp_buffer1, b.st_size) == -1) {close(fd); return -1;};
-    close(fd);
-    compress(COMPRESS_ACTION_COMPRESS, cbuf, (UBYTE *) comp_buffer1, b.st_size,
-	     (UBYTE *) comp_buffer2, &c_len);
-    if (c_len == 0) return -1;			 /* error compressing */
-    if (c_len >= b.st_size) return -2;		 /* compressed size is bigger */
-    fd=creat(wf, S_IREAD|S_IWRITE); if (fd==-1) return -1;
-    if (write(fd, comp_buffer2, c_len)==-1) {close(fd); return -1;}
-    close(fd);
-    return 0;
-}
-    
-
-_errstat exclude_compression(char *fn)
-{
-/* Checks to see if file is one of the files not to compress.
- * 
- * returns 1 if should be excluded (ie. in list)
- * returns 0 if shouldn't be excluded
-*/
-    return exclude_list(fn, exclude_compress);
-}
-
-    
-_s8 stop_compressing;				 /* if 1, then child should terminate */
-void backup_child_term(int signal)
-{
-/* Child has received terminate signal */
-    stop_compressing=1;
-}
-
-void backup_stop_child()
+void backup_stop_child(void)
 {
 /* Stops child from compressing and deletes whatever files
  * are left in temporary buffers */
-    char s[MAXNAMLEN];
+    char s[MAX_FNAME];
     
-    kill(backup_child, SIGTERM);		 /* tell child to stop compressing */
-    waitpid(backup_child, NULL, 0);
-    backup_child = 0;
+    if (backup_child) {
+	kill(backup_child, SIGTERM);		 /* tell child to stop compressing */
+	waitpid(backup_child, NULL, 0);
+	backup_child = 0;
+    }
     while (!feof(fdfifo)) {
 	if (fgets(s, sizeof(s), fdfifo) == NULL) /* original filename */
 	  break;
@@ -182,89 +119,11 @@ void backup_stop_child()
 	unlink(s);				 /* delete it */
     }
     fclose(fdfifo);				 /* close FIFO */
+    fdfifo = NULL;
     unlink(fifo_name);				 /* remove file list & FIFO */
 }
 
 
-void do_backup_child(char *cp)
-{
-/* Loop run in child.
- * 
- * Never returns
-*/
-    long count;
-    char com[MAXNAMLEN], tmpbuf[MAXNAMLEN];    
-    int x=0;
-    
-    close(dv);				 /* child doesn't need tape open */
-    if (log_level > 3) write_log("Backup child successfully forked off");
-    detach_shared_buffers();		 /* child doesn't need any of these */
-    free_non_shared_buffers();
-    if (compression == 2) {
-	if (malloc_comp_buffers() == -1) {
-	    if (log_level > 2) write_log("Unable to make compress 2 buffers - not compressing");
-	    goto finbackup;
-	}
-    }
-    signal(SIGTERM, backup_child_term);	 /* for child to stop compressing */
-    if ((fdfifo = fopen(fifo_name, "w")) == NULL) /* open fifo */
-      goto finbackup;
-    
-    stop_compressing=0;			 /* install TERM handler */
-    cp = info + sizeof(struct info_file_header);   /* loop through */
-    for (count=0;count<ifd.no_in_archive;count++) {   /* and look for files to compress */
-	if (stop_compressing) 		 /* child has received TERM signal */
-	  goto finbackup;
-	if (((struct file_info *) cp)->pos_in_archive == 0)
-	  if (!exclude_compression(cp+sizeof(struct file_info)))/* check we'd compress this */
-	  if (S_ISREG(((struct file_info *) cp)->mode))/* only compress reg files */ {
-	      tmpnam(tmpbuf);				 /* get temp filename */
-	      sprintf(com, " :Compressing %s to %s", cp+sizeof(struct file_info), tmpbuf);
-	      *com = (compression == 1) ? 'E' : 'I';
-	      if (log_level > 2) write_log(com);
-	      switch(compression) {
-	       case 1:		 /* external gzip */
-		  sprintf(com, "%s \"%s\" > %s 2>/dev/null", COMPRESS_PROG, 
-			  (cp+sizeof(struct file_info)), tmpbuf);/* compress file */
-		  change_dollar(com);
-		  x = system(com);
-		  break;
-	       case 2:		 /* internal compress */
-		  x = my_compress(cp+sizeof(struct file_info), tmpbuf);
-		  break;
-	       case 3:		 /* internal gzip */
-		  x = my_gzip(cp+sizeof(struct file_info), tmpbuf);
-		  break;
-		default:
-			x = -1; break;
-	      }
-	      
-	      if (x==0) 		 /* OK compression */
-		x = fprintf(fdfifo, "%s\n%s\n", cp+sizeof(struct file_info), tmpbuf);
-	      else {			 /* problem compressing */
-		  (x==-1) ?
-		    sprintf(com, "I:couldn't compress %s err=%d", cp+sizeof(struct file_info),
-			    errno) :	 /* error copmressing */
-		  sprintf(com, "I:didn't compress %s as it grew", cp+sizeof(struct file_info));
-		  if (log_level > 2)  write_log(com);   /* compressed was larger */
-		  x = fprintf(fdfifo, "%s\n%s\n", cp+sizeof(struct file_info), FIFO_ERR);
-	      }
-	      fflush(fdfifo);		 /* to ensure received */
-	  }
-	cp += sizeof(struct file_info);
-	while (*cp++);
-    }
-    finbackup:;
-    if (fdfifo != NULL) fclose(fdfifo);
-    if (log_level > 3) write_log("Backup child about to finish");
-    free_comp_buffers();
-    my_free_all();
-    detach_shm();
-    free_non_shared_buffers();
-    exit(0);
-}
-
-
 _errstat backup_dpd(char *full_path, struct stat *b)
 {
 /* The routine for each file found by process_dir */
@@ -277,7 +136,6 @@ _errstat backup_dpd(char *full_path, struct stat *b)
     if (S_ISDIR(fi.mode)) 			 /* directories have trailing '/' */
 	if (full_path[strlen(full_path)-1] != '/') 
           strcat(full_path, "/");
-    fi.name_len = strlen(full_path)+1;		 /* recalculate based on above */
     if ( (proc_dev == 0) ||			 /* don't backup if on  */
 	 ((proc_dev == 1) && (proc_dev != dev)) ) {/* /proc device */ 
 	if (add_one_file_engine(NULL, &fi, full_path) != -1)
@@ -287,16 +145,17 @@ _errstat backup_dpd(char *full_path, struct stat *b)
 }
 
 
-inline _errstat add_actual_files(WINDOW *mes_box)
+_errstat add_actual_files(WINDOW *mes_box)
 {
-    long count;
+    _s32 count;
     struct oa_file_entry *se;
     
     for (count=0; count<no_sel; count++) {
 	se = find_entry(sel_files, count);
 	if ( (se->selected == 1) || /* only if directly selected */
-	   (se->selected == 4) )
-	process_dir(NULL, 0, name(se), se->incremental, backup_dpd, TRUE);
+	   (se->selected == 4) ) {
+	    process_dir(NULL, 0, name(se), se->incremental, backup_dpd, TRUE);
+	}
     }
     return 0;
 }
@@ -318,14 +177,14 @@ _errstat write_nameinfo(struct file_info *fi, char *cp)
    Returns 0 if all OK
    Returns -1 if error
 */
-    char l[MAXNAMLEN+50];
+    char l[MAX_FNAME+50];
     
     sprintf(l, "Writing info & filename for %s", cp);
-    if (tape_write((char *) fi, sizeof(struct file_info)) == -1) {
+    if (tape_write_fi(fi) == -1) {
 	write_fatal_log(l);			 /* write info */
 	return -1;
     }
-    if (tape_write((char *) cp, strlen(cp)+1) == -1) {/*  write filename */
+    if (tape_write((char *) cp, fi->name_len) == -1) {/*  write filename */
 	write_fatal_log(l);
 	return -1;
     }
@@ -333,7 +192,7 @@ _errstat write_nameinfo(struct file_info *fi, char *cp)
 }
 
 
-_errstat backup_file(_s32 pos_in_archive)
+_errstat backup_file(WINDOW *mes, _s32 pos_in_archive)
 {
 /* Backup the file pointed to by cp and update cp 
  * An error writing to the backup device is a fatal error
@@ -343,11 +202,13 @@ _errstat backup_file(_s32 pos_in_archive)
 */
     
     struct file_info *fi;
-    char   tmpf[MAXNAMLEN];
-    char   *fn, l[MAXNAMLEN+50];
+    char   tmpf[MAX_FNAME];
+
+    char   *fn, l[MAX_FNAME+50];
     int    fd;
-    _s32   x, totalread;
+    _s32   x, totalread, totalwrite;
     struct stat sbuf, csbuf;
+    WINDOW *m=NULL;
     
     fi = (struct file_info *) cp;		 /* fi points to file info */
     fn = cp + sizeof(struct file_info);
@@ -357,6 +218,8 @@ _errstat backup_file(_s32 pos_in_archive)
 	memset(tmpf, 0, sizeof(tmpf));		 /* name of original file */
 	readlink(fn, tmpf, sizeof(tmpf));	 /* is written */
 	fi->checksum = 0;
+	fi->act_size = strlen(tmpf)+1+sizeof(_s32);
+	fi->size = strlen(tmpf)+1+sizeof(_s32);
 	sprintf(l, "Writing link name for %s [%s]", fn, tmpf);
 	if (log_level > 1) write_log(l);
 	if (write_nameinfo( fi, fn) == -1) {
@@ -364,11 +227,7 @@ _errstat backup_file(_s32 pos_in_archive)
 	    return -1;
 	}
 	x = strlen(tmpf)+1;
-	if (tape_write((char *) &x, sizeof(x)) == -1) {
-	    write_fatal_log(l);
-	    return -1;
-	}
-	if (tape_write((char *) tmpf, x) == -1) {
+	if (tape_write_namelen(tmpf) == -1) {
 	    write_fatal_log(l);
 	    return -1;
 	}
@@ -384,9 +243,10 @@ _errstat backup_file(_s32 pos_in_archive)
 
     *tmpf = 0;					 /* must be a regular file */
     fi->compressed = 0;				 /* not compressed */
-    if ((compression) && (!exclude_compression(fn))) {/* compress if want to and not excluded */
+    if ((compression) && (!exclude_compression(fn)) && (fdfifo != NULL)) {/* compress if want to and not excluded */
 	sprintf(l, "Reading filename from FIFO for %s", fn);
 	if (log_level > 2) write_log(l);
+	f:;
 	if (fgets(tmpf, sizeof(tmpf), fdfifo) == NULL) {   /* gets the original filename */
 	    if (tmpf[strlen(tmpf)-1] == '\n')
 	      tmpf[strlen(tmpf)-1] = 0;
@@ -395,8 +255,17 @@ _errstat backup_file(_s32 pos_in_archive)
 	    *tmpf = 0;
 	    goto non_compress;
 	}
+	if (m) {close_statusbox(m); touchwin(mes); wrefresh(mes);}
 	if (tmpf[strlen(tmpf)-1] == '\n')
 	  tmpf[strlen(tmpf)-1] = 0;
+	if (!strcmp(tmpf, "DISK FULL")) {	 /* child is waiting for disk space */
+	    if (mes) {
+		sprintf(l, "Child has no disk space - waiting");
+		if (log_level > 2) write_log(l);
+		m = status_box(mes, "Child waiting for disk space", 1, TRUE, 1);
+		goto f;				 /* do it again, sam */
+	    }
+	}
 	if (strcmp(tmpf, fn)) {			 /* checks that FIFO & me agree */
 	    sprintf(l, "Mismatch reading in FIFO output for %s [got %s] - saving uncompressed", fn, tmpf);
 	    if (log_level > 2) write_log(l); 
@@ -423,7 +292,7 @@ _errstat backup_file(_s32 pos_in_archive)
 	}
 	sprintf(l, "Opening compressed file %s", tmpf);
 	if (log_level > 2) write_log(l);
-	fd = open(tmpf, O_RDONLY);		 /* open the temp file */
+	fd = open(tmpf, O_RDWR);		 /* open the temp file */
 	if (fd == -1) {
 	    write_error_log(l);
 	    unlink(tmpf);			 /* compressed file, delete it */
@@ -479,24 +348,35 @@ _errstat backup_file(_s32 pos_in_archive)
 	return 0;
     }
     totalread = 0;
+    totalwrite = 0;
     while (1) {					 /* copy file accross */
 	x = read(fd, tr_buffer, max_tr_size);	 /* to device file   */
 	if (!x) break;
 	if (x == -1) {
 	    write_error_log("Error reading file while transferring data to backup device");
-	    while (totalread < fi->act_size) 
-	      totalread += tape_write((char *) tr_buffer, min(max_tr_size,/* pad rubbish data to the end so */
-				       fi->act_size - totalread));
+	    while (totalwrite < fi->act_size) 
+	      totalwrite += tape_write((char *) tr_buffer, min(max_tr_size,/* pad rubbish data to the end so */
+				       fi->act_size - totalwrite));
+	    totalread = totalwrite;
 	    break;				 /* so that the archive integrity is maintained */
 	}
-	totalread += x;				
-	if ((x = tape_write((char *) tr_buffer, x)) == -1) {
-	    close(fd);
-	    return do_exit(ERROR_WRITING);
+	totalread += x;
+	
+	if (totalwrite < fi->act_size) {	 /* don't write too much data to buffer */
+	    if ((x = tape_write((char *) tr_buffer, min(x, fi->act_size-totalwrite))) == -1) {
+		close(fd);
+		return -1;
+	    }
+	    totalwrite += x;
 	}
     }
     close(fd); 
-
+    
+    if (totalread > totalwrite) {
+	sprintf(l, "%s grew while writing. Data at end of file not backed up", fn);
+	write_error_log(l);
+    }
+    
     if (*tmpf)					 /* remove temporary files */
       unlink(tmpf);
     return 0;
@@ -514,7 +394,7 @@ _errstat write_volume_header(_s32 no_files)
     int  c1, sz;
     char *cur_pos;
     struct volume_header *vh1, vh;
-    char null_string;
+    char null_string=0;
 
     if (log_level > 2) write_log("Writing volume header");
     vh.volume_magic = VOLUME_MAGIC;
@@ -525,18 +405,17 @@ _errstat write_volume_header(_s32 no_files)
     sz = sizeof(struct volume_header);
     fe=sel_files;				 /* work out how much space */
     for (c=0; c<no_sel; c++) {			 /* needed for volume header */
-	sz += sizeof(_s32) + strlen(name(fe)) + 1 +
+	sz += sizeof(_s32) + strlen(name(fe))+ 1 +
 	  sizeof(_s32)+ 0 + 1;			 /* room for filter */
 	advance_ofe(&fe);
     }
     vh.size_header = sz;
     ifd.size_volume_headers += sz;		 /* update size */
-    if (tape_write((char *) &vh, sizeof(struct volume_header)) == -1)
-      return -1;
+    if (tape_write_volheader(&vh) == -1) return -1;
     vol_headers = my_realloc(vol_headers, ifd.size_volume_headers);
     if (vol_headers == NULL)
       return do_exit(ERROR_MEMORY);
-    cur_pos = vol_headers;
+    cur_pos = (char *) vol_headers;
     for (c=0; c<ifd.number_volumes-1; c++) {	 /* skip past existing entries */
 	vh1 = (struct volume_header *) cur_pos;	 /* in volume headers */
 	cur_pos += sizeof(struct volume_header);/* past header */
@@ -552,20 +431,16 @@ _errstat write_volume_header(_s32 no_files)
     fe=sel_files;				 /* write file selections */
     for (c=0; c<no_sel; c++) {
 	c1 = strlen(name(fe)) + 1;		 /* file name */
-	if (tape_write((char *) &c1, sizeof(c1)) == -1)
+	if (tape_write_namelen(name(fe)) == -1)
 	  return -1;
-	if (tape_write(name(fe), c1) == -1)
-	  return -1;
-	*((_s32 *) cur_pos) = c1;		 /* filename length */
-	cur_pos += sizeof(_s32);			 /* advance to filename pos */
+	memcpy(cur_pos, &c1, sizeof(_s32));	 /* filename length */
+	cur_pos += sizeof(_s32);		 /* advance to filename pos */
 	strcpy(cur_pos, name(fe));
 	cur_pos += c1;				 /* past filename */
 	c1 = 0+1;				 /* filter */
-	if (tape_write((char *) &c1, sizeof(c1)) == -1)
+	if (tape_write_namelen(&null_string) == -1)
 	  return -1;
-	if (tape_write(&null_string, c1) == -1)	 /* filter */
-	  return -1;
-	*((_s32 *) cur_pos) = c1;
+	memcpy(cur_pos, &c1, sizeof(_s32));
 	cur_pos += sizeof(_s32);
 	strcpy(cur_pos, "");			 /* filter */
 	cur_pos += c1;
@@ -574,41 +449,51 @@ _errstat write_volume_header(_s32 no_files)
     return 0;					 
 }
 
-_errstat write_out_info_file(char *info)
+
+_errstat make_comp_file(char *s)
 {
-    int fd;
+/* Creates a file and gives it a name then writes the names of the files
+ * to be compressed, filename after filename.
+ * 
+ * Returns -1 if error, 0 otherwise
+ */
+    FILE *cf;
+    int c;
+    cp = info + sizeof(struct info_file_header);
+    taper_tmpnam(s);
+    cf = fopen(s, "w+");			 /* create & open file */
+    if (cf == NULL) return -1;
     
-    if ((fd = open_info_file(FALSE, ifd.archive_id)) == -1) /* error message handled */
-	return -1;				 /* by open)info_file */
-    if (log_level > 2) write_log("Writing info file information");
-    if (write(fd, info, ifd.info_file_size) == -1)
-	return do_exit(ERROR_WRITING_INFO);
-    if (write(fd, vol_headers, ifd.size_volume_headers) == -1)   /* write out volume header info */
-      return do_exit(ERROR_WRITING_INFO);
-    if (write(fd, vt_info, ifd.number_volumes*sizeof(struct volume_tape_info)) == -1)
-	return do_exit(ERROR_WRITING_INFO);
-    if (log_level > 2) write_log("Closing info file");
-    close(fd);
+    for (c=0; c<ifd.no_in_archive;c++) {
+	if (((struct file_info *) cp)->pos_in_archive == 0)   /* only if not already in archive */
+	  if (S_ISREG(((struct file_info *) cp)->mode)) {   /* only regular files */
+	      fputs(cp+sizeof(struct file_info), cf);
+	      fputs("\n", cf);
+	  }
+	cp+=sizeof(struct file_info);		 /* advance to next entry */
+	PAST_STRING_FI(cp);
+    }
+    fclose(cf);
     return 0;
 }
 
-
-_errstat  do_backup() {				 
+_errstat  do_backup(void) {				 
 /* For aborting a backup, 
  * 
  * fi.checksum = -2 is written - this means a backup
  *   was aborted. The info file is changed accordingly
 */
-    _s32   count=0, no_old_files, bytes_processed, bytes_written, 
-           no_written, org_files;
+    _s32   count=0, no_old_files, no_written, org_files;
+    _u32   bytes_processed, bytes_written;
     WINDOW *mes=NULL;
-    char   s[100], s2[30], s3[30], tmpbuf[MAXNAMLEN];
+    char   s[100], s2[30], s3[30], s4[30], tmpbuf[MAX_FNAME], cfile[MAX_FNAME];
     time_t t_start, t_current;
     struct volume_tape_info vti;
     int    x;
     _s8    quitting;
     struct file_info *fi;
     char   sa[9][150];
+    struct volume_header v;
 	
     if (!no_sel)				 /* none selected */
       return 0;
@@ -621,7 +506,9 @@ _errstat  do_backup() {
 	tdh.tape_number = 1;			 /* first volume */
 	ifd.archive_id = tdh.archive_id;	 /* setup info file header */
 	ifd.info_file_size = sizeof(struct info_file_header);
-	ifd.number_tapes = 1;
+	ifd.magic = INFO_MAGIC;
+	ifd.number_tapes = 0;
+	ifd.number_tsi = 0;
 	ifd.number_volumes = 0;
 	ifd.size_volume_headers = 0;
 	ifd.no_in_archive = 0;
@@ -632,6 +519,7 @@ _errstat  do_backup() {
     }
     else {
 	ifd = *((struct info_file_header *) info);/* get original ifd */
+	ifd.magic = INFO_MAGIC;			 /* uncompressed originally */
 	no_old_files = ifd.no_in_archive;
 	vt_info = my_realloc(vt_info, sizeof(struct volume_tape_info)*(ifd.number_volumes+1));
     }
@@ -639,15 +527,14 @@ _errstat  do_backup() {
     org_files = no_old_files;
     if (!append) {
 	if (tape_open(O_RDWR) == -1) return -1;
-	blocks_passed = 0;
 	tape_set_blk_size();
 	if (write_tape_header(&tdh) == -1)
 	  return -1;
     }
     else {
-	if (goto_end_vol(mes, 3, ifd.number_volumes, 1, TRUE) == -1)/* goto end of */
+	if (goto_end_vol(mes, 3, ifd.number_volumes, 1, TRUE, TRUE) == -1)/* goto end of */
 	  return -1;				 /* previous volume */
-	blocks_passed = 0;
+	tape_set_blk_size();
     }
 
     ifd.number_volumes++;
@@ -656,13 +543,12 @@ _errstat  do_backup() {
     cp = info + sizeof(struct info_file_header);
     if (append) 				 /* move past existing entries */
       for (count=0; count < no_old_files; count++) {
-	    cp += sizeof(struct file_info);
-	    while (*cp++);
+	  cp += sizeof(struct file_info);
+	  PAST_STRING_FI(cp);
       }
 
     if (add_actual_files(mes) == -1)
       return -1;				 /* add filenames to directory */
-
     
     memcpy(info, &ifd, sizeof(struct info_file_header));
     status_box(mes, "", 3, FALSE, 1);
@@ -679,45 +565,78 @@ _errstat  do_backup() {
     bytes_written=0;
     bytes_processed=0;
     print_kb(s3, total_selected);
-    if (compression) {
-	tmpnam(fifo_name);
-	if ((mknod(fifo_name, S_IFIFO|S_IREAD|S_IWRITE, 0)) == -1) 
+    fdfifo = NULL;
+    *cfile=0;
+    if (compression) 
+      if (make_comp_file(cfile) != -1) {
+	  taper_tmpnam(fifo_name);
+	  if ((mknod(fifo_name, S_IFIFO|S_IREAD|S_IWRITE, 0)) == -1) 
 	    return do_exit(ERROR_CREATING_FIFO);
-	if (backup_child) {				 /* a process still going on */
-	    kill(backup_child, SIGKILL);
-	    waitpid(backup_child, NULL, 0);
-	    backup_child = 0;
-	}
-	if (log_level > 3) write_log("About to fork of backup child");
-	backup_child = fork();
-	if (backup_child == 0) {		 /* we are in child process */
-	    do_backup_child(cp);		 /* never returns from here */
-	}
+	  if (backup_child) {				 /* a process still going on */
+	      kill(backup_child, SIGKILL);
+	      waitpid(backup_child, NULL, 0);
+	      backup_child = 0;
+	  }
+	  if (log_level > 3) write_log("About to fork of backup child");
+	  backup_child = fork();
+	  if (backup_child == 0) {		 /* we are in child process */
+	      close(dv);
+	      chdir(original_cur_dir);
+#ifdef TRIPLE_BUFFER
+	      sprintf(tmpbuf, "B:Executing bg_backup %s %d %s %s %d %d \"%s\" %s %d",
+		      log_file, log_level, cfile, fifo_name, compression, shm_id, exclude_compress, temp_dir, min_free);
 
-	if (backup_child == -1) {
-	    unlink(fifo_name); 
-	    backup_child = 0;
-	    return do_exit(ERROR_UNABLE_FORK);
-	}
-	if ((fdfifo = fopen(fifo_name, "r")) == NULL) {
-	    kill(backup_child, SIGKILL);		 /* kill child process  */
-	    waitpid(backup_child, NULL, 0);
-	    backup_child = 0;
-	    return do_exit(ERROR_OPENING_FIFO);
-	}
-	if (comp_head_start) {			 /* let the compression program */
-	    status_box(mes, "Giving compression a head start", 3, FALSE, 1);
-	    sleep(60*comp_head_start);		 /* have a head start */
-	    t_start = time(NULL);		 /* restart timer */
-	}
-    }
+	      sprintf(s2, "%d", shm_id);
+#else
+	      sprintf(tmpbuf, "B:Executing bg_backup %s %d %s %s %d %d \"%s\" %s %d",
+			log_file, log_level, cfile, fifo_name, compression, 0, exclude_compress, temp_dir, min_free);
+	      sprintf(s2, "%d", 0);
+#endif
+	      if (log_level > 3) write_log(tmpbuf);
+	      sprintf(s, "%d", compression);
+	      sprintf(s3, "%d", log_level);
+	      sprintf(s4, "%d", min_free);
+	      if (execlp("bg_backup", "bg_backup", log_file, s3, cfile, fifo_name, s, s2, exclude_compress, temp_dir, s4, NULL) == -1) {		  write_error_log("starting background backup");
+		  fdfifo = fopen(fifo_name, "w");
+		  fprintf(fdfifo, "NO CHILD\n"); /* no child! */
+		  fclose(fdfifo);
+		  exit(-1);			 /* should never get here */
+	      }
+	  }
+	  
+	  if (backup_child == -1) {
+	      unlink(fifo_name); 
+	      if (*cfile) unlink(cfile);
+	      backup_child = 0;
+	      return do_exit(ERROR_UNABLE_FORK);
+	  }
+	  if ((fdfifo = fopen(fifo_name, "r")) == NULL) {
+	      if (backup_child) {
+		  kill(backup_child, SIGKILL);		 /* kill child process  */
+		  waitpid(backup_child, NULL, 0);
+		  backup_child = 0;
+	      }
+	      return do_exit(ERROR_OPENING_FIFO);
+	  }
+	  fgets(tmpbuf, sizeof(tmpbuf), fdfifo);
+	  tmpbuf[strlen(tmpbuf)-1] = 0;		 /* remove trailing '\n' */
+	  if(!strcmp(tmpbuf, "NO CHILD")) {
+	      waitpid(backup_child, NULL, WNOHANG);
+	      return do_exit(ERROR_NO_BACKUP_CHILD);
+	  }
+	  if (comp_head_start) {			 /* let the compression program */
+	      status_box(mes, "Giving compression a head start", 3, FALSE, 1);
+	      sleep(60*comp_head_start);		 /* have a head start */
+	      t_start = time(NULL);		 /* restart timer */
+	  }
+      }
     cp = info + sizeof(struct info_file_header);
-    nodelay(mes, TRUE);
+    if (mes) nodelay(mes, TRUE);
     quitting=0;
     no_written=0;
     t_start = time(NULL);
     for (count=0; count< ifd.no_in_archive; count++) {
-	*s = wgetch(mes);
+	*s = (mes == NULL) ? 0 : wgetch(mes);
 	if (((*s == 'q') || (*s == 'Q')) && (!quitting)) {
 	    if (message_box("Confirm abort backup", MB_YESNO)) {
 		touchwin(mes); wrefresh(mes);
@@ -728,7 +647,11 @@ _errstat  do_backup() {
 		fi->checksum = -2;
 		write_nameinfo(fi, "");
 		ifd.no_in_archive = no_old_files;/* correct info file */
-		get_vh(vol_headers, ifd.number_volumes)->no_in_volume = no_written; 
+		memcpy(&v, get_vh(vol_headers, ifd.number_volumes), sizeof(struct volume_header));   /* this needed to */
+		v.no_in_volume = no_written;	 /* ensure aligned access */
+		memcpy(get_vh(vol_headers, ifd.number_volumes), &v, sizeof(struct volume_header));   
+		write_log("Backup aborted by user");
+		log_errors++;
 		break;
 	    }
 	}
@@ -769,21 +692,27 @@ _errstat  do_backup() {
 			  print_mb(s2, bytes_processed/x*60));
 		status_box(mes, s, 8, FALSE, 1);
 	    }
-	    if (backup_file(++no_old_files) == -1) {
-		tape_close(dv);			 /* rewinding device so will auto update headers */
+	    if (!no_windows) {
+		touchwin(mes); wrefresh(mes);	 /* in case new tape */
+	    }
+	    if (backup_file(mes, ++no_old_files) == -1) {
+		tape_close();			 /* rewinding device so will auto update headers */
 		backup_stop_child();
+		if (*cfile) unlink(cfile);
+		waitpid(backup_child, NULL, WNOHANG);
 		return -1;			 /* error message handled by backup file */
 	    }
-	    bytes_processed += sizeof(struct file_info) + 1 + strlen(cp+sizeof(struct file_info));
-	    bytes_written += sizeof(struct file_info) + 1 + strlen(cp+sizeof(struct file_info));
-	    if (S_ISREG(((struct file_info *) cp)->mode)) {   /* only add this if regular file */
+	    bytes_processed += sizeof(struct file_info) + ((struct file_info *) cp)->name_len;
+	    bytes_written += sizeof(struct file_info) + ((struct file_info *) cp)->name_len;
+	    if ((S_ISREG(((struct file_info *) cp)->mode)) ||
+		(S_ISLNK(((struct file_info *) cp)->mode)) ) {   /* only add this if regular file */
 		bytes_processed += ((struct file_info *) cp)->size;
 		bytes_written += ((struct file_info *) cp)->act_size;
 	    }
 	}
 	
 	cp += sizeof(struct file_info);
-	while (*cp++);
+	PAST_STRING_FI(cp);
     }
     t_current = time(NULL);
     x = t_current - t_start;
@@ -793,42 +722,54 @@ _errstat  do_backup() {
     status_box(mes, "", 4, FALSE, 1);
     status_box(mes, "", 6, FALSE, 1);
     status_box(mes, "Updating header segments", 3, FALSE, 1);
-    tape_close(dv);				 /* rewinding device so will auto update headers */
-    memcpy(info, &ifd, sizeof(struct info_file_header));
+    tape_close();				 /* rewinding device so will auto update headers */
     vti.blocks_on_last_tape = blocks_passed;
     vti.end_tape = ifd.number_tapes;
     memcpy(vt_info+ifd.number_volumes-1, &vti, sizeof(vti));   
-    write_out_info_file(info);
-    fclose(fdfifo);				 /* close FIFO */
-    unlink(fifo_name);				 /* remove file list & FIFO */
+/* Code for info file at end here */
+    if (write_out_info_file(info, ifd.archive_id) != -1)
+      if (compress_info) compress_info_file(ifd.archive_id);
+    if (fdfifo) {
+      fclose(fdfifo);				 /* close FIFO */
+      unlink(fifo_name);				 /* remove file list & FIFO */
+    }
+    if (*cfile) unlink(cfile);			 /* in case child didn't */
     waitpid(backup_child, NULL, 0);
     backup_child = 0;
     close_statusbox(mes);
-    touchwin(win_main); wrefresh(win_main);
+    if (win_main) {
+	touchwin(win_main); 
+	wrefresh(win_main);
+    }
     sprintf(sa[0], "Backup Finished");
     strcpy(sa[1], "");
     sprintf(sa[2], "Backed up: %d files,  %s  [%s]", ifd.no_in_archive - org_files,
 	    print_mb(s2, bytes_written),
 	    print_mb(s3, bytes_processed));
     if (log_level > 1) write_log(sa[2]);
-    sprintf(sa[3], "Total on archive %s [%s]. Ratio %.2f", 
+    if (mf) {write(mf, sa[2], strlen(sa[2])); write(mf, "\n", 1);}
+        sprintf(sa[3], "Total on archive %s [%s]. Ratio %.2f", 
 	    print_mb(s2, bytes_written+total_compressed),
 	    print_mb(s3, bytes_processed+total_uncompressed),
 	    (total_compressed+bytes_written == 0) ? 1 : 
 	       (float) (total_uncompressed+bytes_processed)/
 	       (float) (total_compressed+bytes_written));
     if (log_level > 1) write_log(sa[3]);
+    if (mf) {write(mf, sa[3], strlen(sa[3])); write(mf, "\n", 1);}
     strcpy(sa[4], "");
     sprintf(sa[5], "Time elapsed %s.", convtime(s2, t_start, t_current));
     if (log_level > 1) write_log(sa[5]);
+    if (mf) {write(mf, sa[5], strlen(sa[5])); write(mf, "\n", 1);}
     sprintf(sa[6], "Backup rate %s/min [%s/min]", 
 	    print_mb(tmpbuf, bytes_written/x*60),
 	    print_mb(s3, bytes_processed/x*60));
     if (log_level > 1) write_log(sa[6]);
+    if (mf) {write(mf, sa[6], strlen(sa[6])); write(mf, "\n", 1);}
     strcpy(sa[7], "");
     sprintf(sa[8], "%d warnings, %d errors", log_warnings, log_errors);
     if (log_level > 1) write_log(sa[8]);
-    multi_message_box(sa, 9, MB_OK);
+    if (mf) {write(mf, sa[8], strlen(sa[8])); write(mf, "\n", 1);}
+    if (!no_windows) multi_message_box(sa, 9, MB_OK);
     return 0;
 }
 
@@ -837,22 +778,12 @@ _errstat read_in_fileset(int argc, char *argv[])
 {
 /* Reads in the files */
     FILE *f;
-    char ln[MAXNAMLEN];
+    char ln[MAX_FNAME], s[MAX_FNAME];
     struct oa_file_entry *ce;
-    _s32 c;
+    _s32 c, c1;
     struct direntry entry;
     time_t t;
     
-    tmpnam(mailfn);
-    mf = open(mailfn, O_CREAT|O_RDWR, S_IREAD|S_IWRITE);
-    if (mf==-1) 
-      return do_exit(ERROR_OPENING_MAIL);
-    
-    time(&t);
-    sprintf(ln, "To: %s\n", MAIL_TO); write(mf, ln, strlen(ln));
-    sprintf(ln, "Date: %s", ctime(&t)), write(mf, ln, strlen(ln));
-    strcpy(ln, "Subject: Taper unattended backup\n"); write(mf, ln, strlen(ln));
-
     c = 0;
     while (c < argc) {
 	if ( (!strcmp(argv[c], "-U") ||
@@ -881,12 +812,12 @@ _errstat read_in_fileset(int argc, char *argv[])
 			    ln[strlen(ln)-1] = 0;
 			  if (*ln) {
 			      ce=sel_files;
-			      for (c=0; c<no_sel; c++) {		 /* make sure not already in */
+			      for (c1=0; c1<no_sel; c1++) {		 /* make sure not already in */
 				  if (!strcmp(name(ce), ln)) 
 				    break;
 				  advance_ofe(&ce);
 			      }
-			      if (c==no_sel)
+			      if (c1==no_sel)
 				if (get_statinfo(ln, &entry.info) != -1) {/* ignore if error */
 				    select_entry_1(&entry, ln);
 				    write(mf, "  Selected ", strlen("  Selected "));
@@ -895,8 +826,8 @@ _errstat read_in_fileset(int argc, char *argv[])
 				    write(mf, ln, strlen(ln));
 				}
 			      else {
-				  sprintf(ln, "WARNING: %s is not found\n", &argv[c][1]);
-				  write(mf, ln, strlen(ln));
+				  sprintf(s, "WARNING: %s is not found in fileset %s\n", ln, &argv[c][1]);
+				  write(mf, s, strlen(s));
 			      }
 			  }
 			  fgets(ln, sizeof(ln), f);		 /* ignore filter */
@@ -918,20 +849,21 @@ _errstat read_in_fileset(int argc, char *argv[])
 	c++;
     }
     time(&t);
-    sprintf(ln, "\nBackup commenced at %s", ctime(&t));
+    sprintf(ln, "\nBackup commenced at %s\n", ctime(&t));
     write(mf, ln, strlen(ln));
     return 1;
 }
 
    
 _errstat utils_write_nullheader(WINDOW **mes);
-_errstat check_backup() {
+_errstat check_backup(void) {
 /* Make sure backup is a valid device and confirm with use about
    an erase if it has been requested.         */
     int x;
     WINDOW *mes=NULL;
-    char info_file[MAXNAMLEN];
+    char info_file[MAX_FNAME];
     struct stat buf;
+    _s32 oldarch;
 
     if (stat(tape, &buf) == -1) {		 /* touch file if needed */
 	if (tape_type == TAPE_TYPE_FILE) {
@@ -952,6 +884,10 @@ _errstat check_backup() {
 	}
 	else {					 /* unattended mode */
 	    if (!tape_overwrite) {
+		if (mf) write(mf, "Unknown data on tape. Tape not overwritten.\n", 
+			      strlen("Unknown data on tape. Tape not overwritten.\n"));
+		if (mf) write(mf, "Use --tape-overwrite-on [+O] to force overwriting\n",
+		              strlen("Use --tape-overwrite-on [+O] to force overwriting\n"));
 		write_warning_log("Unknown data on the tape. Tape not overwritten");
 		return 0;	 /* need to have authority to overwrite */
 	    }
@@ -979,12 +915,15 @@ _errstat check_backup() {
 	    unlink(tape);				 /* regular files get deleted */
 	    close(creat(tape, S_IREAD|S_IWRITE));/* and re-created to get zero length */
 	}
-	else 
+	else {
+	  oldarch = tdh.archive_id;
 	    utils_write_nullheader(&mes);
+	    tape_close();
+	}
 	strcpy(info_file, taper_info_files);	 /* look for information file */
 	if (info_file[strlen(info_file)-1] != '/')	 /* associated with this archive */
 	  strcat(info_file, "/");
-	sprintf(info_file, "%staper_info.%u", info_file, tdh.archive_id);
+	sprintf(info_file, "%staper_info.%u", info_file, oldarch);
 	unlink(info_file);			 /* attempt to delete info file */
 	if (mes) 
 	  close_statusbox(mes);
@@ -998,7 +937,7 @@ _errstat check_backup() {
     }
     
     return 1;
-   }
+}
     
 
 
@@ -1006,15 +945,23 @@ void taper_backup(int argc, char *argv[])
 {
     int      x;
     int      old_append;
-    char     s[MAXNAMLEN];
 
     old_append = append;			 /* save value of append for future use */
     if (open_logfile("Backup") == -1)
       return;
     if (!no_windows) {
-	backrest_init_windows() ;			 /* Draw windows */
+	backrest_init_windows() ;		 /* Draw windows */
 	backrest_clear_screen();
     }
+    else {					 /* open file to mail things to */
+	taper_tmpnam(mailfn);
+	mf = open(mailfn, O_CREAT|O_RDWR, S_IREAD|S_IWRITE);
+	if (mf==-1) {
+	    do_exit(ERROR_OPENING_MAIL);
+	    return;
+	}
+    }
+    
     if (check_device_names() == -1) goto endbackup;/* check devices & other parms */
     if (backrest_do_mallocs() == -1)		 /* couldn't allocate memory */
       goto endbackup;
@@ -1046,6 +993,8 @@ void taper_backup(int argc, char *argv[])
 	backrest_kill_windows();
 	print_my_name();
 	if (x > 0)  do_backup();
+	if (dv > 0) tape_close();
+	
     }
     else {
 	backrest_kill_windows();
@@ -1054,15 +1003,10 @@ void taper_backup(int argc, char *argv[])
 
     backrest_free_memory();
 
-    if (mf) {					 /* if mail file */
-	mail_finish("Backup");
-	close(mf);
-	sprintf(s, "%s %s < %s", MAIL_PROG, MAIL_TO, mailfn);   /* send mail */
-	system(s);
-	unlink(mailfn);
-    }
+    if (mf) 
+	  sendmail(); 					 /* if mail file */
     append = old_append;
-    
+   
     endbackup:;
     close_logfile("Backup");
 }
