@@ -1,7 +1,15 @@
 /*
-   Time-stamp: <95/12/07 00:52:11 yusuf>
+   Time-stamp: <96/07/19 20:17:46 yusuf>
+
+   $Id: memory.c,v 1.6 1996/07/27 20:42:10 yusuf Exp $	
+
 */
 
+#ifndef lint
+static char vcid[] = "$Id: memory.c,v 1.6 1996/07/27 20:42:10 yusuf Exp $";
+#endif /* lint */
+
+
 
 /* My memory routines. Maintains a list of currently allocated
  * blocks and provides a way to delete them all in one go. Can
@@ -57,9 +65,11 @@ void my_free(void *block) {
     for (c=0; c<MAX_ALLOCED; c++)		 /* look for block */
       if (alloced[c] == block)
         break;
-    free(block);				 /* free block */
-    if (c < MAX_ALLOCED)
-      alloced[c] = NULL;			 /* remove from my list */
+    if (c < MAX_ALLOCED) {
+    	free(block);				 /* free block */
+    	alloced[c] = NULL;			 /* remove from my list */
+    }
+    else exit(-10);
 }
 
 void my_free_all() {
@@ -70,3 +80,4 @@ void my_free_all() {
         my_free(alloced[c]);
 }
 
+
