@@ -1,7 +1,15 @@
 /*
-   Time-stamp: <95/12/07 00:50:29 yusuf>
+   Time-stamp: <96/07/19 20:18:18 yusuf>
+
+   $Id: sel_restore.c,v 1.23 1996/07/27 20:42:12 yusuf Exp $	
+
 */
 
+#ifndef lint
+static char vcid[] = "$Id: sel_restore.c,v 1.23 1996/07/27 20:42:12 yusuf Exp $";
+#endif /* lint */
+
+
 
 #include "taper.h"
 #include "restore.h"
@@ -27,7 +35,7 @@
 */
 
 extern select_details file_sd, select_sd, vol_sd;
-char start_path[MAXNAMLEN];
+char start_path[MAX_FNAME];
 
 int lfmr_cmp(void const *key, void const *s)
 {
@@ -57,9 +65,11 @@ struct oa_file_entry *look_most_recent(struct oa_file_entry *fe,
  */
 
     struct oa_file_entry *f, *old_f, *cur;
+    char s[MAX_FNAME];
 
     if (starts == NULL) { 			 /* we have to find first entry that matches */
-	f = (struct oa_file_entry *) search_file(name(fe), (char *) archive_files, fe->i.volume, 0);
+	strcpy(s, cf); strcat(s, name(fe));	 /* make full pathname for search file */
+	f = (struct oa_file_entry *) search_file(s, fe->i.volume);
 	starts = f;
 	if (f == NULL) return fe;			 /* problem - couldn't find it */
     }
@@ -94,7 +104,7 @@ void restore_asterix_line(WINDOW *win, int ent_num, int line)
 }
 
 
-void restore_asterix_dir(WINDOW *win, long start, char *p_scroll) {
+void restore_asterix_dir(WINDOW *win, _s32 start, char *p_scroll) {
     int    cur_line=1, cur_ent;
     
     cur_ent = start;
@@ -107,7 +117,7 @@ void restore_asterix_dir(WINDOW *win, long start, char *p_scroll) {
 }
 
 
-void restore_print_voldir_line(WINDOW *win, long entry, int line, char ref) {
+void restore_print_voldir_line(WINDOW *win, _s32 entry, int line, char ref) {
 /*  Prints one entry in a directory at line 'line'  */
 
     char   s[NAME_MAX+1], s2[50];
@@ -132,7 +142,7 @@ void restore_print_voldir_line(WINDOW *win, long entry, int line, char ref) {
 	  strcat(s, " +");
 	else
 	  strcat(s, "  ");
-	sprintf(s, "%s%-*s", s, left_width-strlen(s)-1, fe->name);
+	sprintf(s, "%s%-*s", s, left_width-(int) strlen(s)-1, fe->name);
 	t = *localtime(&fe->o.i.backup_time);
 	if (fe->o.i.checksum == -1)			 /* error backing up */
 	  strcpy(s2, "BACK ERROR");
@@ -156,9 +166,9 @@ void restore_print_voldir_line(WINDOW *win, long entry, int line, char ref) {
       wrefresh(win);
 }
 
-void restore_print_vol_dir(WINDOW *win, long start, char *p_scroll) {
+void restore_print_vol_dir(WINDOW *win, _s32 start, char *p_scroll) {
     int    cur_line=1, cur_ent;
-    char   s[50], s1[MAXNAMLEN], s2[MAXNAMLEN];
+    char   s[50], s1[MAX_FNAME], s2[MAX_FNAME];
     
     my_werase(win, COLOR_DIRECTORY);
     *p_scroll = 0;
@@ -199,18 +209,28 @@ void restore_print_vol_dir(WINDOW *win, long start, char *p_scroll) {
 }
 
     
-void restore_print_selected_line(WINDOW *win, long entry, int line, char ref) {
+void restore_print_selected_line(WINDOW *win, _s32 entry, int line, char ref) {
     
     struct oa_file_entry *mr;
     char   s1[50];
     
     mr = find_entry(sel_files, entry);
-    (mr->selected == 1) ?  mvwprintw(win, line+1, 5, " %9s  %s ", 
-				     print_kb(s1,(mr->incremental == 1) ? mr->mm_size : mr->ab_size),
+    if (((mr->incremental == 1) ? mr->mm_size : mr->ab_size) < 1024) {
+    (mr->selected == 1) ?  mvwprintw(win, line+1, 5, " %8d   %s ", 
+				     (mr->incremental == 1) ? mr->mm_size : mr->ab_size,
 				     name(mr)) :
-                           mvwprintw(win, line+1, 5, "(%9s  %s)",  
-				     print_kb(s1,(mr->incremental == 1) ? mr->mm_size : mr->ab_size),
+                           mvwprintw(win, line+1, 5, "(%8d   %s)",  
+				     (mr->incremental == 1) ? mr->mm_size : mr->ab_size,
 				     name(mr));
+    }
+    else {
+	(mr->selected == 1) ?  mvwprintw(win, line+1, 5, " %9s  %s ", 
+					 print_kb(s1,(mr->incremental == 1) ? mr->mm_size : mr->ab_size),
+					 name(mr)) :
+	                       mvwprintw(win, line+1, 5, "(%9s  %s)",  
+		                         print_kb(s1,(mr->incremental == 1) ? mr->mm_size : mr->ab_size),
+		                         name(mr));
+    }
     (mr->incremental == 1) ? mvwaddstr(win, line+1, win->_maxx-5, "M  ") :
                              mvwprintw(win, line+1, win->_maxx-5, "%03d", abs(mr->i.volume));
     wattroff(win, A_DIM);
@@ -231,8 +251,8 @@ void restore_print_selected_size(WINDOW *win)
 }
 
 
-void  restore_print_selected(WINDOW *win, long top_sel, char *p_scroll) {
-    long   cur;
+void  restore_print_selected(WINDOW *win, _s32 top_sel, char *p_scroll) {
+    _s32   cur;
     
     my_werase(win, COLOR_SELECTED);
     restore_print_selected_size(win);
@@ -248,7 +268,7 @@ void  restore_print_selected(WINDOW *win, long top_sel, char *p_scroll) {
 }
 
 
-void restore_look_for_indirect()
+void restore_look_for_indirect(void)
 {
 /* Goes through the restore selected files and looks for any
    selections that become duplicated.
@@ -257,7 +277,7 @@ void restore_look_for_indirect()
    and then you selected /usr/bin/games, /usr/bin/games/tetris should
    become an indirect selection
 */
-    long c, c1;
+    _s32 c, c1;
     struct oa_file_entry *fe, *f;
 
     fe = sel_files;
@@ -281,7 +301,7 @@ void restore_look_for_indirect()
 }
 
 
-void tag_appropriate(long num)
+void tag_appropriate(_s32 num)
 {
 /* Tags the appropriate file in archive files for sel_restore.
    Takes into consideration whether selecting actual file or most recent file
@@ -290,7 +310,7 @@ void tag_appropriate(long num)
  * the size of the tagged files is also updated
 */
     struct oa_file_entry *f, *g, *fe, *sfe, *starts;
-    long c=0;
+    _s32 c=0;
     int i;
     WINDOW *mes=NULL;
 
@@ -354,11 +374,11 @@ void tag_appropriate(long num)
 }
 
 
-void restore_update_selected()
+void restore_update_selected(void)
 {
 /* Goes through archive and works out indirect selections */
 
-    long c;
+    _s32 c;
     struct oa_file_entry *cur_ent;
     struct dir_file_entry *d;
 
@@ -395,7 +415,7 @@ void restore_update_selected()
 }
 
 
-int toggle_selection(long cur_entry)
+int toggle_selection(_s32 cur_entry)
 {
 /* Toggles the currently selected item from most recent restore to volume
    restore & vice-versa
@@ -423,16 +443,16 @@ void del_sel_engine(struct oa_file_entry *fe)
     restore_asterix_dir(files, file_sd.top_entry, &dummy);
 }
 
-void restore_del_cur_entry(select_details *sd, long *no_sel)
+void restore_del_cur_entry(select_details *sd, _s32 *no_sel)
 {
     del_sel_engine(find_entry(sel_files, sd->cur_entry));
 }
 
 
-void restore_unselect_dir(select_details *sd, long *dummy)
+void restore_unselect_dir(select_details *sd, _s32 *dummy)
 {
     struct oa_file_entry *fe, *fe1;
-    long c;
+    _s32 c;
 
     fe = &(dirs + sd->cur_entry-1)->o;
     fe1=sel_files;
@@ -498,7 +518,7 @@ int tag_voldir_0(struct oa_file_entry *orgfe)
 }
 
 
-int tag_voldir_line(long cur_entry) {
+int tag_voldir_line(_s32 cur_entry) {
 /* Selects the file at cur_entry.
  * It then checks through the whole file directory looking
  * if any other files should also be selected (eg. if you
@@ -550,7 +570,7 @@ void make_mydir(int vol)
 /* If volume is non-zero, then only selects those files which belong
  * to vol volume */
 /* Makes a directory with the path pointed to by start_path */
-    long c=0;
+    _s32 c=0;
     struct oa_file_entry *fe=archive_files;
     char *s, *x;
 
@@ -578,15 +598,15 @@ void make_mydir(int vol)
     }
     in_dir++;					 /* up dir */
     clear_sd(&file_sd);
-    qsort(dirs, in_dir-1, sizeof(struct dir_file_entry), rsdr);
+    qsort(dirs, in_dir-1, sizeof(struct dir_file_entry), rsdr); 
 }
 
     
-void restore_restore_backupset()
+void restore_restore_backupset(void)
 {
-    char ln[MAXNAMLEN], s[MAXNAMLEN];
+    char ln[MAX_FNAME], s[MAX_FNAME];
     FILE *f;
-    long c;
+    _s32 c;
     struct oa_file_entry *ce;
 
     f = backrest_restore_backupset(NULL);
@@ -619,10 +639,10 @@ void restore_restore_backupset()
     fclose(f);
 }
 
-int restore_on_vol_select(long entry)
+int restore_on_vol_select(_s32 entry)
 {
 /* Selects entry for restore */
-    long c=0;
+    _s32 c=0;
     char *s;
     int  lv=0;
     struct oa_file_entry *fa;
@@ -640,23 +660,22 @@ int restore_on_vol_select(long entry)
 	!strncmp(s, "Backed up at ", strlen("Backed up at ")) ) 
 	    return 0;
     
-    fa = (struct oa_file_entry *) search_file(s, (char *) archive_files, lv, 0);/* couldn't find it in archive */
+    fa = (struct oa_file_entry *) search_file(s, lv);/* couldn't find it in archive */
     if (fa == NULL) return 0;
     return tag_voldir_0(fa);
 }
 
 
-int select_restore_files() 
+int select_restore_files(void) 
 {
 /* Returns 0 if quit
  * Returns 1 if wants to restore
 */
     int ret, oldvol;
-    struct info_file_header *ifd;
     struct dir_file_entry *s;
     char p;
-    char old_start[MAXNAMLEN];
-    long c;
+    char old_start[MAX_FNAME];
+    _s32 c;
 
     *start_path = 0;
     clear_sd(&file_sd);
@@ -668,7 +687,6 @@ int select_restore_files()
     p=0;
     restore_print_selected(selection, select_sd.top_entry, &p);
     
-    ifd = (struct info_file_header *) info;
     make_mydir(only_vol);
     while (1) {
 	ret = select_box(files, &in_dir, &file_sd,
@@ -778,5 +796,6 @@ int select_restore_files()
 	    break;
 	}
     }
+    return 0; /* never get here */
 }
     
