@@ -1,7 +1,16 @@
 /*
-   Time-stamp: <95/12/07 00:51:00 yusuf>
+   Time-stamp: <96/08/05 19:32:56 yusuf>
+
+   $Id: taper.c,v 1.27 1996/08/05 19:02:03 yusuf Exp $	
+
 */
 
+#ifndef lint
+static char vcid[] = "$Id: taper.c,v 1.27 1996/08/05 19:02:03 yusuf Exp $";
+#endif /* lint */
+
+
+
 /* This is the main Taper file.
 
 
@@ -13,10 +22,7 @@
 #include "taper.h"
 
 
-PUBLIC char cur_dir[MAXNAMLEN];
-
-
-PRIVATE void usage()
+PRIVATE void usage(void)
 {
 /* Prints usage of common options */   
     fprintf(stderr, "\n\n");
@@ -30,6 +36,7 @@ PRIVATE void usage()
     fprintf(stderr, "+/- a   (--append-[on|off])       turns on/off append\n");
     fprintf(stderr, "-A id   (--archive-diff)          prints differences between archive & disk\n");
     fprintf(stderr, "- c num (--compress-type)         compression type (0 none, 1 e gzip, 2 int comp, 3 int gzip, 4 fast comp\n");
+    fprintf(stderr, "+/- C   (--bad-checksum-[on|off]) prompt if encounter bad checksum\n");
     fprintf(stderr, "-d id   (--print-dir)             prints the directory of archive id\n");
     fprintf(stderr, "+/- D   (--auto-descend-[on|off]) automatically ascend/descend directories\n");
     fprintf(stderr, "-F file (--exclude-files)         which files not to include in archive\n");
@@ -38,7 +45,9 @@ PRIVATE void usage()
     fprintf(stderr, "+/- h   (--hard-links-[on|off])   turns on/off storing of links or actual files\n");
     fprintf(stderr, "-H min  (--comp-head-start]       number of minutes head start for compressor\n");
     fprintf(stderr, "-i path (--info-files)            where the archive information files are\n");
+    fprintf(stderr, "+/- I   (--compress-info-[on|off])turns on/off compression of info files\n");
     fprintf(stderr, "-j num  (--proc-device num)       device number of /proc system\n");
+/*    fprintf(stderr, "+/- J   (--info-at-end-[on|off])  turns on/off info file at end of archive\n");    not yet */
     fprintf(stderr, "-l file (--log-file)              name of log file\n");
     fprintf(stderr, "-m num  (--log-level)             level of logging (0=none..4=all)\n");
     fprintf(stderr, "+/-M    (--memory-tight-[on|off]) limit memory use\n");
@@ -49,15 +58,16 @@ PRIVATE void usage()
     fprintf(stderr, "+/-Q    (--fast-fsf-[on|off])     tape drive support fast fsf\n");
     fprintf(stderr, "-r path (--restore-path)          where to restore files to\n");
     fprintf(stderr, "-R list (--exclude-dirs)          which directories to automatically exclude\n");
-    fprintf(stderr, "-s num  (--strip)                 level of pathname stripping\n");
+    fprintf(stderr, "-s num  (--strip-paths)           level of pathname stripping\n");
     fprintf(stderr, "+/-S    (--set-blksize-[on|off])  Set block size before writing\n");
     fprintf(stderr, "-t name (--archive-title)         title of archive\n");
     fprintf(stderr, "+/- u   (--incremental-[on|off])  whether to use incremental backups by default\n");
     fprintf(stderr, "-v      (--version)               prints current version\n");
     fprintf(stderr, "-x num  (--block-size)            size of block to write to tape with\n");
     fprintf(stderr, "-X file (--exclude-compress)      which files not to compress\n");
-    fprintf(stderr, "+/- y   (--alpha-sort-[on|off])   turns on or off directory sorting\n");
+    fprintf(stderr, "+/- y   (--sort-dirs-[on|off])    turns on or off directory sorting\n");
     fprintf(stderr, "-/+ z   (--prompt-dirs-[on|off])  confirm directory selection\n");
+    fprintf(stderr, "-Z num  (--tape-size)             size of tape in MB\n");
     fprintf(stderr, "-U file (--unattended-file)       do an unattended backup with file (set)\n");   /*  */
     fprintf(stderr, "+/- w   (--recent-restore-[on|off]  most recent file restore or selected file\n");
     fprintf(stderr, "-?      (--help)                  help\n");
@@ -90,6 +100,28 @@ PUBLIC char *make_tt(char tt)
 }
 
 
+PUBLIC char make_short_tt(char tt)
+{
+/* Makes a char  of tape type depending on tt. 
+*/
+    switch(tt) {
+     case TAPE_TYPE_ZFTAPE:
+	return 'z';
+     case TAPE_TYPE_SCSI:
+	return 's';
+     case TAPE_TYPE_FTAPE:
+	return 'f';
+     case TAPE_TYPE_FILE:
+	return 'l';
+     case TAPE_TYPE_REMOVABLE:
+	return 'r';
+     case TAPE_TYPE_IDE:
+	return 'i';
+    }
+    return 0;
+}
+
+
 PRIVATE _errstat set_tape_type(char *s)
 {
 /* Sets tape_type depending on what tape is given in s. 
@@ -118,7 +150,7 @@ PRIVATE _errstat set_tape_type(char *s)
     return tape_type;
 }
 
-PRIVATE void read_environment()
+PRIVATE void read_environment(void)
 {
     if (getenv(TAPE_TYPE_ENVIRON))
       if (set_tape_type(getenv(TAPE_TYPE_ENVIRON)) == -1)/* set tape type from environ */
@@ -251,7 +283,7 @@ PRIVATE void process_pref_line(char *s)
     while (ap[c].pref_name) {
 	if (!ap[c].save_pref) {		 /* this preference may have leading type */
 	    if (!strcasecmp(&s[2], &ap[c].pref_name[2])) {   /* matches - must check prefix */
-		if (*s == ((tape_type == -1) ? 'f' : *make_tt(tape_type)))
+		if (*s == ((tape_type == -1) ? 'f' : make_short_tt(tape_type)))
 		  goto found_option;
 		else 
 		  return;			 /* ignore this line */
@@ -297,7 +329,7 @@ PRIVATE void process_pref_line(char *s)
 }
 
 
-PRIVATE inline _errstat check_a(int argc, int *c)
+PRIVATE _errstat check_a(int argc, int *c)
 {
 /* Increments c and makes sure it is less than argc 
  * 1 if OK, 0 if not */
@@ -340,7 +372,7 @@ PRIVATE void read_prefs_file(int argc, char *argv[])
 	    if (!check_a(argc, &c)) {usage(); do_exit(ERROR_USAGE);}
 	    if (set_tape_type(argv[c]) == -1)
 	      do_exit(ERROR_ILLEGAL_TAPE_TYPE);
-	    break;
+	    continue;
 	}
 	c++;
     }
@@ -455,6 +487,7 @@ PRIVATE void parse_args(int argc, char *argv[])
 		  no_windows = 1;		 /* tell not to open windows */
 		  break;
 	       case 'U':			 /* Unattended files */
+		  if (!check_a(argc, &c)) goto usage;
 		  no_windows = 1;
 		  unattended = 1;
 		  break;
@@ -491,20 +524,6 @@ PUBLIC _errstat fill_in_defaults()
  * Returns -1 if error, 0 otherwise
  
  */
-    int d;
-    
-    if (tape_type == -1) {			 /* see if we */
-	d = open(tape, O_RDONLY);		 /* are dealing with  */
-	if (d == -1) {				 /* a regular file */
-	    if (errno == ENOENT)			 /* file doesn't exist */
-	      tape_type = TAPE_TYPE_FILE;
-	}
-	else {
-	    if (is_regfile(d))
-	      tape_type = TAPE_TYPE_FILE;
-	    close(d);
-	}
-    }
     switch(tape_type) {
      case -1:
 	return do_exit(ERROR_NO_TAPE_TYPE);
@@ -514,6 +533,7 @@ PUBLIC _errstat fill_in_defaults()
 	if (*tape == -1) strcpy(tape, FTAPE_DEFAULT_TAPE);
 	if (*ntape == -1) strcpy(ntape, FTAPE_DEFAULT_NTAPE);
 	if (fast_fsf == -1) fast_fsf = 0;
+	if (can_seek == -1) can_seek = 0;
 	if (set_blksize == -1) set_blksize = 0;
 	if (get_blksize == -1) get_blksize = 0;
 	have_rewind = 1;
@@ -527,6 +547,7 @@ PUBLIC _errstat fill_in_defaults()
 	if (fast_fsf == -1) fast_fsf = 1;
 	if (set_blksize == -1) set_blksize = 1;
 	if (get_blksize == -1) get_blksize = 1;
+	if (can_seek == -1) can_seek = 1;
 	have_rewind = 1;
 	have_fsf = 1;
 	break;
@@ -536,6 +557,7 @@ PUBLIC _errstat fill_in_defaults()
 	if (*tape == -1) strcpy(tape, SCSI_DEFAULT_TAPE);
 	if (*ntape == -1) strcpy(ntape, SCSI_DEFAULT_NTAPE);
 	if (fast_fsf == -1) fast_fsf = 1;
+	if (can_seek == -1) can_seek = 1;
 	if (set_blksize == -1) set_blksize = 0;
 	if (get_blksize == -1) get_blksize = 0;
 	have_rewind = 1;
@@ -546,6 +568,7 @@ PUBLIC _errstat fill_in_defaults()
 	if (block_size == -1) block_size = FILE_DEFAULT_BLOCK_SIZE;
 	if (*tape == -1) strcpy(tape, "");
 	if (*ntape == -1) strcpy(ntape, "");
+	can_seek = 1;
 	fast_fsf = 0;
 	set_blksize = 0;
 	get_blksize = 0;
@@ -558,6 +581,7 @@ PUBLIC _errstat fill_in_defaults()
 	if (*tape == -1) strcpy(tape, REMOVABLE_DEFAULT_TAPE);
 	if (*ntape == -1) strcpy(ntape, REMOVABLE_DEFAULT_NTAPE);
 	fast_fsf = 0;
+	if (can_seek == -1) can_seek = 0;
 	set_blksize = 0;
 	get_blksize = 0;
 	have_rewind = 0;
@@ -569,6 +593,7 @@ PUBLIC _errstat fill_in_defaults()
 	if (*tape == -1) strcpy(tape, IDE_DEFAULT_TAPE);
 	if (*ntape == -1) strcpy(ntape, IDE_DEFAULT_NTAPE);
 	if (fast_fsf == -1) fast_fsf = 1;
+	if (can_seek == -1) can_seek = 0;
 	if (set_blksize == -1) set_blksize = 0;
 	if (get_blksize == -1) get_blksize = 0;
 	have_rewind = 1;
@@ -579,7 +604,7 @@ PUBLIC _errstat fill_in_defaults()
 }
 
 
-PRIVATE void setup_colors()
+PRIVATE void setup_colors(void)
 {
 /* Sets up color */
     
@@ -807,7 +832,7 @@ PRIVATE int do_form_driver(FORM *form, FIELD *fields[], int c, int cf, int count
     int k, ff=0;
     
     for (k=0; k<cf; k++) 			 /* convert cf into field on form */
-      if (options[k].type == 'S')
+      if ((options[k].type == 'S') || (options[k].type == 'N'))
         ff++;
     if (!(current_field(form) == fields[ff]))
       set_current_field(form, fields[ff]);
@@ -852,8 +877,9 @@ PUBLIC _errstat taper_change_prefs(struct pref_info *options, _s8 allab)
     WINDOW *help;
     FIELD *fields[30];
     FORM *form;
+    char  tmp[100];
+    int   *intp;
 
-    my_werase(win_main, COLOR_FORM); 
     count=0;					 /* count number options */
     while (options[count].name) {
 	count++;
@@ -861,31 +887,42 @@ PUBLIC _errstat taper_change_prefs(struct pref_info *options, _s8 allab)
     
     top = (win_main->_maxy - (count * 2)) / 2;	 /* print screen template */
     k1=0;
-    for (c=0; c<count;c++) 			 /* now setup forms fields */
+    for (c=0; c<count;c++) {			 /* now setup forms fields */
 	if (options[c].type == 'S') {
-	    fields[k1] = new_field(1, 40, top+c*2, 33, 0, 0);
+	    fields[k1] = new_field(1, 40, top+c*2, 33, 15, 0);
 	    field_opts_off(fields[k1], O_STATIC); 
 	    set_field_back(fields[k1], COLOR_PAIR(COLOR_FORM)|A_REVERSE);
-	    field_opts_off(fields[k1], O_AUTOSKIP);
 	    set_max_field(fields[k1], 0); 
 	    set_field_buffer(fields[k1], 0, options[c].opt);
 	    k1++;
 	}
+	if (options[c].type == 'N') {
+	    fields[k1] = new_field(1, 40, top+c*2, 33, 1, 0);
+	    field_opts_off(fields[k1], O_STATIC); 
+	    set_field_back(fields[k1], COLOR_PAIR(COLOR_FORM)|A_REVERSE);
+	    set_max_field(fields[k1], 0); 
+	    intp = options[c].opt;		 /* this done to avoid */
+	    sprintf(tmp, "%d", *intp);		 /* warnings */
+	    set_field_buffer(fields[k1], 0, tmp);
+	    k1++;
+	}
+    }
 
     fields[k1] = NULL;
     form=new_form(fields);
-    set_form_win(form, win_main);
-    set_form_sub(form, win_main);
+    set_form_win(form, win_main);  
+    my_werase(win_main, COLOR_FORM); 
     post_form(form);
     print_option_screen(options, top, count, allab);
     c=0;
     print_pref_line(options, c, top, 1);
-    curs_set((options[c].type == 'S') ? 1: 0);
-    wmove(win_main, top+c*2, 33);
+    curs_set( ((options[c].type == 'S') || (options[c].type == 'N')) ? 1: 0);
     wrefresh(win_main);
+    wmove(win_main, top+c*2, 33);
     while (1) {
 	k = wgetch(win_main);
-	if ((options[c].type == 'S') && (k != EXIT_KEY))
+	if ( ((options[c].type == 'S') || (options[c].type == 'N')) && 
+	    (k != EXIT_KEY))
 	  if (do_form_driver(form, fields, k, c, count, options) == -1)
 	    continue;				 /* processed key */
 	print_pref_line(options, c, top, 0);
@@ -899,19 +936,26 @@ PUBLIC _errstat taper_change_prefs(struct pref_info *options, _s8 allab)
 	    c--; 
 	    break;
 	 case ABORT_KEY:
-	    if (allab)
-	      return -1;
+	    if (allab) return -1;
 	    break;
 	 case EXIT_KEY:
 	    form_driver(form, REQ_VALIDATION);
 	    k=0;
 	    for (c=0; c<count;c++) {		 /* copy form */
-		if (options[c].type == 'S') {	 /* information into */
+		switch(options[c].type) {
+		 case 'S':
 		    my_strcpy(options[c].opt, field_buffer(fields[k], 0), 
 			      options[c].length);   /* appropriate vars */
 		    if (!strcmp(options[c].opt, " "))/* if space only */
 		      *(char *) options[c].opt = 0;  /* make NULL */
 		    k++;
+		    break;
+		 case 'N':
+		    my_strcpy(tmp, field_buffer(fields[k], 0), sizeof(tmp));	 /* appropriate vars */
+		    intp = options[c].opt;
+		    *intp = atoi(tmp);
+		    k++;
+		    break;
 		}
 	    }
 	    unpost_form(form);
@@ -944,16 +988,18 @@ PUBLIC _errstat taper_change_prefs(struct pref_info *options, _s8 allab)
 	      wgetch(help);
 	      my_delwin(help);
 	      print_option_screen(options, top, count, allab);
+	      touchwin(win_main); wrefresh(win_main);
 	      break;
-	  }
+	}
 	if (c<0) c=count-1;
 	if (c==count) c= 0;
 	
-	curs_set((options[c].type == 'S') ? 1: 0);
+	curs_set( ((options[c].type == 'S') || (options[c].type == 'N')) ? 1: 0);
 	wmove(win_main, top+c*2, 33);
 	print_pref_line(options, c, top, 1);
 	wrefresh(win_main);
     }
+    return -1;                         /* Should never get here */
 }
 
 
@@ -979,17 +1025,15 @@ PRIVATE char *oo(_s32 o)
 }
 
 
-PRIVATE void taper_save_preference_file()
+PRIVATE void taper_save_preference_file(void)
 {
-    struct pref_info o[] = { {"Preference file", pref_file, 'S', MAXNAMLEN},
+    struct pref_info o[] = { {"Preference file", pref_file, 'S', MAX_FNAME},
 			     {NULL, NULL, 0}
 		       };
     FILE *pf;
-    char s[MAXNAMLEN];
+    char s[MAX_FNAME];
     int c=0;
     char tp;
-    int  gbs, sbs, et, ff, bs;
-    char t[MAXNAMLEN], nt[MAXNAMLEN];
     
     if (taper_change_prefs(o, 1) == -1)
       return;
@@ -1005,64 +1049,43 @@ PRIVATE void taper_save_preference_file()
 	do_exit(ERROR_CREATING_PREFS);
 	return;
     }
-/* Save these values only if different to defaults */    
-    gbs=get_blksize; get_blksize=-1;	 /* save current values */
-    sbs=set_blksize; set_blksize=-1;
-    ff=fast_fsf; fast_fsf=-1;
-    et=erase_tape; erase_tape=-1;
-    bs=block_size; block_size =-1;
-    strcpy(t, tape); *tape = -1;
-    strcpy(nt, ntape); *ntape = -1;
-    fill_in_defaults();
     tp = *make_tt(tape_type);
-    if (gbs != get_blksize) {
-	fprintf(pf, "%c-get-blksize = %s\n", tp, tf(gbs));
-	get_blksize = gbs;
-    }
-    if (sbs != set_blksize) {
-	fprintf(pf, "%c-set-blksize = %s\n", tp, tf(sbs));
-	set_blksize = sbs;
-    }
-    if (ff != fast_fsf) {
-	fprintf(pf, "%c-fast-fsf = %s\n", tp, tf(ff));
-	fast_fsf = ff;
-    }
-    if (et != erase_tape) {
-	fprintf(pf, "%c-erase-tape = %s\n", tp, tf(et));
-	erase_tape = et;
-    }
-    if (bs != block_size) {
-	fprintf(pf, "%c-block-size = %d\n", tp, bs);
-	block_size = bs;
-    }
-    if (strcmp(t, tape)) {
-	fprintf(pf, "%c-tape = %s\n", tp, t);
-	strcpy(tape, t);
-    }
-    
-    if (strcmp(nt, ntape)) {
-	fprintf(pf, "%c-tape = %s\n", tp, nt);
-	strcpy(ntape, nt);
-    }
     
     if (ap[c].save_pref) {
 	while (ap[c].pref_name) {
 	    switch(ap[c].handle) {
 	     case 'S':
-		if (ap[c].cln != 'b')
-		  fprintf(pf, "%s = \"%s\"\n", &ap[c].pref_name[2], (char *) ap[c].option_1);
+		if (ap[c].cln != 'b') {
+		    if (ap[c].save_pref)
+		      fprintf(pf, "%s = \"%s\"\n", &ap[c].pref_name[2], (char *) ap[c].option_1);
+		    else
+		      fprintf(pf, "%c-%s = \"%s\"\n", tp, &ap[c].pref_name[2], (char *) ap[c].option_1);
+		}
 		break;
 	     case 'I':
-		fprintf(pf, "%s = %d\n", &ap[c].pref_name[2], *(_s32 *) ap[c].option_1);
+	     case 'L':
+		if (ap[c].save_pref)
+		  fprintf(pf, "%s = %d\n", &ap[c].pref_name[2], *(_s32 *) ap[c].option_1);
+		else
+		  fprintf(pf, "%c-%s = %d\n", tp, &ap[c].pref_name[2], *(_s32 *) ap[c].option_1);
 		break;
 	     case 'Y':
-		fprintf(pf, "%s = %s\n", &ap[c].pref_name[2], tf(*(char *) ap[c].option_1));
+		if (ap[c].save_pref) 
+		  fprintf(pf, "%s = %s\n", &ap[c].pref_name[2], tf(*(char *) ap[c].option_1));
+		else
+		  fprintf(pf, "%c-%s = %s\n", tp, &ap[c].pref_name[2], tf(*(char *) ap[c].option_1));
 		break;
 	     case 'T':
-		fprintf(pf, "%s = %s\n", &ap[c].pref_name[2], make_tt(*(char *) ap[c].option_1));
+		if (ap[c].save_pref)
+		  fprintf(pf, "%s = %s\n", &ap[c].pref_name[2], make_tt(*(char *) ap[c].option_1));
+		else
+		  fprintf(pf, "%c-%s = %s\n", tp, &ap[c].pref_name[2], make_tt(*(char *) ap[c].option_1));
 		break;
 	     case 'C':
-		fprintf(pf, "%s = %s\n", &ap[c].pref_name[2], color_string(*(char *) ap[c].option_1, *(char *) ap[c].option_2));
+		if (ap[c].save_pref)
+		  fprintf(pf, "%s = %s\n", &ap[c].pref_name[2], color_string(*(char *) ap[c].option_1, *(char *) ap[c].option_2));
+		else
+		  fprintf(pf, "%c-%s = %s\n", tp, &ap[c].pref_name[2], color_string(*(char *) ap[c].option_1, *(char *) ap[c].option_2));
 		break;
 	    }
 	c++;
@@ -1072,10 +1095,10 @@ PRIVATE void taper_save_preference_file()
 }
 
 
-PRIVATE void taper_save_command_line()
+PRIVATE void taper_save_command_line(void)
 {
-    char s[MAXNAMLEN];
-    struct pref_info o[] = { {"Command line file", cf, 'S', MAXNAMLEN},
+    char s[MAX_FNAME];
+    struct pref_info o[] = { {"Command line file", cf, 'S', MAX_FNAME},
 			     {NULL, NULL, 0}
 		       };
     FILE *pf;
@@ -1102,6 +1125,7 @@ PRIVATE void taper_save_command_line()
 	      fprintf(pf, "%s \"%s\"  ", ap[c].pref_name, (char *) ap[c].option_1);
 	    break;
 	 case 'I':
+	 case 'L':
 	    fprintf(pf, "%s %d  ", ap[c].pref_name, *(int *) ap[c].option_1);
 	    break;
 	 case 'Y':
@@ -1121,7 +1145,8 @@ PRIVATE void taper_save_command_line()
 }
 
 
-PRIVATE void taper_init_windows() {
+PRIVATE void taper_init_windows(void)
+{
 /* Initialise main window and print screen edges etc..  */
 
     WINDOW *whole_screen;
@@ -1147,40 +1172,66 @@ PRIVATE void taper_init_windows() {
 }
 
 typedef void (*sighandler_t)(int);		 /* makes for clearer def */
-sighandler_t old_sig;
     
-PRIVATE void my_sig(int sig)
+PRIVATE void my_int_sig(int sig)
 /* Handles the INT signal. Basically doesn't allow the user to interrupt -
    interrupting while backups are in progress is just too risky
 */
 {
-    signal(sig, my_sig);
+    signal(sig, my_int_sig);			 /* Linux restores orig signal behavious */
 }
 
 
-PRIVATE void preferences_menu()
+PRIVATE void my_sf_sig(int sig)
+{
+/* Handes a seg fault */
+    signal(sig, SIG_DFL);			 /* to avoid recursion if further seg faults occur */
+    my_werase(win_main, COLOR_MAIN); wmove(win_main, 1,1); wrefresh(win_main);
+    endwin();
+    curs_set(1);
+    if (sig == SIGSEGV)
+      fprintf(stderr, "taper: segmentation fault.\nAttempting to clean up\n");
+    else
+      fprintf(stderr, "taper: abort signal.\nAttempting to clean up\n");
+    my_free_all();
+    if (shm)
+      if (write_pid) kill(write_pid, SIGTERM);	 /* old write child still going */
+    if (backup_child) kill(backup_child, SIGTERM);
+    if (restore_child) kill(restore_child, SIGTERM);
+    free_buffers(); 
+    (sig == SIGSEGV) ?  write_log("ERROR: Segmentation fault") :
+                       write_log("ERROR: Abort signal");
+    write_log("\n\n");
+    exit(-1);
+}
+
+
+PRIVATE void preferences_menu(void)
 {
     int in_op=0;
     char *menu[] = {
       "Tape drive preferences", "Global preferences", 
-	"Backup preferences",
-	"Restore preferences", "Unattended preferences",
+	"Backup preferences 1", "Backup preferences 2",
+	"Restore/Mkinfo preferences", "Unattended preferences",
 	"Back to main menu", NULL};
     struct pref_info tapedrive_options[] = {
 /*	{"Tape drive tape", &tape_type, 'T', 0}, */
-	{"Rewinding device", tape, 'S', MAXNAMLEN},
-	{"Non-rewinding device", ntape, 'S', MAXNAMLEN},
+	{"Rewinding device", tape, 'S', MAX_FNAME},
+	{"Non-rewinding device", ntape, 'S', MAX_FNAME},
 	{"Have fast fsf", &fast_fsf, 'Y', 0},
+	{"Can seek", &can_seek, 'Y', 0},  
 	{"Set block size", &set_blksize, 'Y', 0},
 	{"Get block size", &get_blksize, 'Y', 0},
 	{"Block size", &block_size, 'B', 0},
 	{"Need to erase tapes", &erase_tape, 'Y', 0},
+	{"Tape size in MB", &tape_size, 'N', 0},
 	{NULL, NULL, 0, 0}
     };
     struct pref_info global_options[] = { 	
 	{"Memory tight", &memory_tight, 'Y', 0},
 	{"Info file directory", taper_info_files, 'S', MAX_ARCHIVETITLE},
-	{"Log file", log_file, 'S', MAXNAMLEN},
+	{"Temporary files directory", temp_dir, 'S', MAX_FNAME},  
+	{"Log file", log_file, 'S', MAX_FNAME},
 	{"Log level", &log_level, 'L', 0},
 	{"Max log size", &log_file_size, 'I', 0},
 	{"Strip path", &ostrip, 'I', 0},
@@ -1189,24 +1240,33 @@ PRIVATE void preferences_menu()
 	{"/proc device", &proc_dev, 'I', 0},
 	{NULL, NULL, 0, 0}
     };
-    
-    struct pref_info backup_options[] = { {"Archive title", archive_title, 'S', MAX_ARCHIVETITLE},
+    struct pref_info backup_options_1[] = { {"Archive title", archive_title, 'S', MAX_ARCHIVETITLE},
 	{"Volume title", volume_title, 'S', MAX_ARCHIVETITLE},
-	{"Exclude compress", &exclude_compress, 'S', MAXNAMLEN},
-	{"Exclude files", &exclude_files, 'S', MAXNAMLEN},
-        {"Exclude dirs", &exclude_dirs, 'S', MAXNAMLEN},
-	{"Hard links", &hard_links, 'Y', 0},
+	{"Exclude compress", &exclude_compress, 'S', MAX_FNAME},
+	{"Exclude files", &exclude_files, 'S', MAX_FNAME},
+        {"Exclude dirs", &exclude_dirs, 'S', MAX_FNAME},
 	{"Compress", &compression, 'C', 0},
 	{"Compression head start", &comp_head_start, 'I', 0},
-	{"Default incremental backup", &incremental, 'Y', 0},
+	{"Hard links", &hard_links, 'Y', 0},
 	{NULL, NULL, 0, 0}
     };
-    struct pref_info restore_options[]= { {"Restore path", rel_path, 'S', MAXNAMLEN},
+    struct pref_info backup_options_2[] = { 
+	{"Compress info files", &compress_info, 'Y', 0},  
+	{"Limit to one filesystem", &ofs, 'Y', 0},  
+	{"Default incremental backup", &incremental, 'Y', 0},
+	{"Free space for compress", &min_free, 'N', 0},  
+/*	{"Write info at end of tape", &info_end, 'Y', 0}, not yet */
+	{NULL, NULL, 0, 0}
+    };
+    struct pref_info restore_options[]= { {"Restore path", rel_path, 'S', MAX_FNAME},
 	{"Overwrite level", &ovrwrite, 'O', 0},
 	{"Only volume", &only_vol, 'I', 0},
 	{"Prompt for archives", &prompt_archive, 'Y', 0},
 	{"Default recent restore", &most_recent, 'Y', 0},
 	{"Auto descend directories", &auto_descend, 'Y', 0},
+	{"Prompt if bad checksum", &bad_checksum, 'Y', 0},
+	{"Min blocks before seeking", &min_seek, 'I', 0},
+	{"Using pre-5.6 archive", &old_archive, 'Y', 0},  
 	{NULL, NULL, 0, 0}
     };
     
@@ -1227,33 +1287,38 @@ PRIVATE void preferences_menu()
 	    taper_change_prefs(global_options, 0);
 	    break;
 	 case 2:
-	    taper_change_prefs(backup_options, 0);
+	    taper_change_prefs(backup_options_1, 0);
 	    break;
 	 case 3:
-	    taper_change_prefs(restore_options, 0);
+	    taper_change_prefs(backup_options_2, 0);
 	    break;
 	 case 4:
+	    taper_change_prefs(restore_options, 0);
+	    break;
+	 case 5:
 	    taper_change_prefs(unattended_options, 0);
 	    break;
-	 case 5:					 
+	 case 6:					 
 	    return;
 	}
     }
 }
 
 
-void utils_mktape();
-void utils_testzero();
-void utils_testfast_fsf();
-void utils_whereproc();
-void utils_erasevols();
-void utils_test_mktape();
-void utils_testlinks();
-PRIVATE void utilities()
+void utils_mktape(void);
+void utils_testzero(void);
+void utils_testfast_fsf(void);
+void utils_whereproc(void);
+void utils_erasevols(void);
+void utils_test_mktape(void);
+void utils_test_can_seek(void);
+void utils_testlinks(void);
+PRIVATE void utilities(void)
 {
     char *menu[] = {
-	"Make tape",  "Test make tape", "Where's proc", 
-	  "Test fast fsf", "Test leading zeroes", 
+	"Make tape",  "Where's proc", 
+	  "Test fast fsf", "Test make tape",
+	  "Test can seek",
 	  "Erase volumes", "Look for recursive links",
 	  "Back to main menu", NULL};
     int in_op=0;
@@ -1265,16 +1330,16 @@ PRIVATE void utilities()
 	    utils_mktape();
 	    break;
 	 case 1:
-	    utils_test_mktape();
-	    break;
-	 case 2:
 	    utils_whereproc();
 	    break;
-	 case 3:					 /* global options */
+	 case 2:					 /* global options */
 	    utils_testfast_fsf();
 	    break;
-	 case 4:					 /* global options */
-	    utils_testzero();
+	 case 3:
+	    utils_test_mktape();
+	    break;
+	 case 4:
+	    utils_test_can_seek();
 	    break;
 	 case 5:
 	    utils_erasevols();
@@ -1290,24 +1355,45 @@ PRIVATE void utilities()
 }
 
 
+PRIVATE void save_prefs(void)
+{
+    char *menu[] = {
+	"Save to preference file", "Save to command line file",
+	"Return to main menu",  NULL};
+    int in_op=0;
+    
+    while (1) {
+	clear_main();
+	switch (select_menu(win_main, menu, &in_op)) {
+	 case 0:
+	    taper_save_preference_file();
+	    break;
+	 case 1:
+	    taper_save_command_line();
+	    break;
+	 case 2:
+	    return;
+	}
+    }
+}
+
+	    
 void taper_backup(int, char **);
-void taper_restore();
-void taper_mkinfo();
+void taper_restore(void);
+void taper_verify(void);
+void taper_mkinfo(void);
 int main(int argc, char *argv[]) 
 {
 
   int fin=0, in_op=0;
   char *menu[] = {
-      "Backup Module", "Restore Module", "Make Info/Check archive",
-	"Utilities", "Change Preferences", "Save preferences to preference file",
-	"Save preferences to command line", "Exit", NULL};
+      "Backup Module", "Restore Module", "Recreate info file",
+	"Verify archive", 
+	"Utilities", "Change Preferences", "Save preferences", 
+	"Exit", NULL};
 
-  sighandler_t old_sig;
 
-  write_buffer = NULL;
   read_buffer = NULL;
-  w_buffer_1 = NULL;
-  w_buffer_2 = NULL;
   tr_buffer = NULL;
   shm = NULL;
   win_main = NULL;
@@ -1320,7 +1406,9 @@ int main(int argc, char *argv[])
   if (init_buffers(0) == -1) return -1;
   org_block_size=block_size;	                 /* save original block size */
   if (!no_windows) taper_init_windows();	 /* Draw windows */
-  old_sig = signal(SIGINT, my_sig);  		 /* trap INT signal */
+  signal(SIGINT, my_int_sig);			 /* trap INT signal */
+  signal(SIGSEGV, my_sf_sig);			 /* trap seg fault signal */
+  signal(SIGABRT, my_sf_sig);			 /* trap seg fault signal */
   getcwd(cur_dir, sizeof(cur_dir));		 /* save current directory for backup */
   getcwd(original_cur_dir, sizeof(original_cur_dir)); /* for restore */
   if (no_windows) {
@@ -1334,28 +1422,29 @@ int main(int argc, char *argv[])
 	  clear_main();
 	  init_common_vars();			 /* set all vars */
 	  switch (select_menu(win_main, menu, &in_op)) {
-	     case 0:					 /* backup */
+	   case 0:					 /* backup */
 	      taper_backup(argc, argv);
 	      break;
-	     case 1:					 /* restore */
+	   case 1:					 /* restore */
+	      restore_mode = RESTORE_FULL;
 	      taper_restore();
 	      break;
-	     case 2:					 /* mkinfo */
+	   case 2:					 /* mkinfo */
 	      taper_mkinfo();
 	      break;
 	   case 3:
-	      utilities();
+	      taper_verify();
 	      break;
 	   case 4:
+	      utilities();
+	      break;
+	   case 5:
 	      block_size=org_block_size;	 /* save original block size */
 	      preferences_menu();
 	      org_block_size = block_size;
 	      break;
-	 case 5:
-	    taper_save_preference_file();
-	    break;
-	 case 6:
-	    taper_save_command_line();
+	   case 6:
+	    save_prefs();
 	    break;
 	   case 7:					 /* exit */
 	      fin=1;
@@ -1382,6 +1471,8 @@ int main(int argc, char *argv[])
     if (shm)
       if (write_pid) kill(write_pid, SIGTERM);	 /* old write child still going */
     free_buffers();
-    signal(SIGINT, old_sig);			 /* restore INT signal */
+    signal(SIGINT, SIG_DFL);			 /* restore signals */
+    signal(SIGSEGV, SIG_DFL);
+    signal(SIGABRT, SIG_DFL);
     return 0;			                 /* succesful return */
 }
