@@ -1,19 +1,23 @@
 /*
-   Time-stamp: <95/12/07 00:51:05 yusuf>
+   Time-stamp: <96/07/19 20:18:46 yusuf>
+
+   $Id: taper.h,v 1.9 1996/07/27 20:42:17 yusuf Exp $
 */
 
 
 /* Functions in tapeio.h */
 
+#define __FROM_MAIN__
+
+
 #include "version.h"
 
-#define inline __inline__ 			/* inline is not an ANSI keyword */
-
-#include <stdio.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <ctype.h>
 #include <dirent.h>
+#include <errno.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <utime.h>
@@ -24,18 +28,19 @@
 #include <sys/types.h>
 #include <sys/ipc.h>
 #include <sys/shm.h>
+#include <sys/sem.h>
 #include <sys/mount.h>
 #include <sys/wait.h>
 #include <curses.h>
 #include <form.h>
 
-#include "errors.h"
-#include "defaults.h"
-#include "structs.h"
+#include "select_box.h"
 #include "compress/lzrw3.h"
 #include "compress/gzip.h"
-#include "select_box.h"
-#include "memory.h"
+#include "non-ansi.h"
+#include "defaults.h"
+#include "structs.h"
 #include "vars.h"
 #include "common.h"
 #include "tapeio.h"
+#include "endianize.h"
