@@ -140,7 +140,7 @@ int  compression=2;				/* Default is internal compression */
 int  sort_dir=1;
 char archive_title[MAX_ARCHIVETITLE]={0};
 char volume_title[MAX_ARCHIVETITLE]={0};
-char exclude_files[MAX_FNAME]={".o ~"};		 /* files to exclude in archive */
+char exclude_files[MAX_FNAME]={".o ~ core"};		 /* files to exclude in archive */
 char exclude_dirs[MAX_FNAME]={"/tmp /usr/tmp /var/tmp"};/* directories to exclude in archive */
 int  unattended=0;				 /* for unattended back sets */
 char rel_path[MAX_FNAME]={0};
